// TTGO_Aprs v1.00 : Programme TTGO LoRa.
// Ecrit par Frank LOBA.
// Date création : 22/03/2023.
// Dernière mise à jour : 22/03/2023.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <SPI.h>
#include <LoRa.h>

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>


/**** DEFINITION DES CONSTANTES ****/
#define OLED_SDA 21
#define OLED_SCL 22 
#define OLED_RST 16

#define OLED_SCREEN_WIDTH   128
#define OLED_SCREEN_HEIGHT  64

#define OLED_ADRESS 0x3C


#define LORA_SCK  5
#define LORA_SS   18
#define LORA_MISO 19  
#define LORA_MOSI 27
#define LORA_RST  23
#define LORA_DIO0 26

#define LORA_FREQUENCE         433775000L
#define LORA_SPREADING_FACTOR  12
#define LORA_SIGNAL_BANDWIDTH  125000L
#define LORA_CODING_RATE_4     5
#define LORA_SYNC_WORD         0x12

#define LORA_START_BYTE_1 '<'
#define LORA_START_BYTE_2 0xFF
#define LORA_START_BYTE_3 0x01


/**** DEFINITION DES VARIABLES GLOBALES ****/
Adafruit_SSD1306 Display(OLED_SCREEN_WIDTH, OLED_SCREEN_HEIGHT, &Wire, OLED_RST);

String Trame;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation de l'interface série (USB).
  Serial.begin(115200);

  // Initialisation de l'écran OLED.
  Wire.begin(OLED_SDA, OLED_SCL);
  
  if(!Display.begin(SSD1306_SWITCHCAPVCC, OLED_ADRESS, false, false))
  {
    // Ecriture d'un message d'erreur dans le port série en cas d'échec.
    Serial.println("Echec initialisation ecran");
    
    while(1);
  }

  // Effacement de l'écran, définition de la couleur et de la taille du texte.
  Display.clearDisplay();
  Display.setTextColor(WHITE);
  Display.setTextSize(1);

  // Affectation des broches pour LoRa.
  SPI.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_SS);
  LoRa.setPins(LORA_SS, LORA_RST, LORA_DIO0);


  // Défintion de la fréquence et démarrage du LoRa.
  if(!LoRa.begin(LORA_FREQUENCE))
  {
    // Ecriture d'un message d'erreur dans le port série en cas d'échec.
    Serial.println("Echec demarrage LoRa");

    // Affichage d'un message à l'écran en cas d'échec.
    Display.setCursor(0,0);
    Display.println("Echec demarrage LoRa");
    Display.display();
    
    while (1);
  }

  // Définition des paramètres système pour LoRa.
  LoRa.setSpreadingFactor(LORA_SPREADING_FACTOR);
  LoRa.setSignalBandwidth(LORA_SIGNAL_BANDWIDTH);
  LoRa.setCodingRate4(LORA_CODING_RATE_4);
  LoRa.enableCrc();
  LoRa.setSyncWord(LORA_SYNC_WORD);

  // Ecriture d'un message dans le port série en cas de succès.
  Serial.println("LoRa OK");

  // Affichage d'un message à l'écran en cas de succès.
  Display.setCursor(0,0);
  Display.println("LoRa OK");
  Display.display();  
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  // Détection de la réception d'un paquet.
  int PacketSize = LoRa.parsePacket();
  if (PacketSize)
  {
    // Effacement de l'écran.
    Display.clearDisplay();

    // Positionnement du curseur.
    Display.setCursor(0,0);

    // Affichage de "RX" à l'écran.
    Display.println("**** RX ****");

    // Initialisation de la trame TNC2.
    Trame = "";

    // Détection de l'en-tête LoRa.
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_1)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_2)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_3)
    
    // Lecture du paquet au format TNC2.
    while(LoRa.available())
    Trame += (char)LoRa.read();

    // Affichage de la trame au format TNC2 à l'écran.
    Display.println(Trame);
    Display.display();
  }

  delay(20);
}
